==> jsmex-trader-sdk-1.1.32.tgz: package/lib/data/KlineDTO.js <==
const IDTO = global.IDTO || require('./IDTO')
const Joi = global.Joi || require('../../static_dependencies/joi/joi-browser.min.js')
const UID = global.UID || require('../utils/UID')

/**
 * KlineDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class KlineDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.symbolId = ''
    this.resolution = '1'
    this.time = 0

    this.klineId = `${this.symbolId}-${this.resolution}-${this.time}`

    this.open = ''
    this.close = ''
    this.high = ''
    this.low = ''
    this.volume = 0
    this.deal = 0
  }

  /**
   * KlineDTO get symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
   * @return {String}
  */
  get symbol () {
    return this.symbolId
  }

  /**
   * KlineDTO set symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
   * @param {String} symbolId
  */
  set symbol (symbolId) {
    this.symbolId = symbolId
  }

  /**
   * KlineDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    this.klineId = `${this.symbolId}-${this.resolution}-${this.time}`
    let data = {
      symbolId: this.symbolId,
      resolution: this.resolution,
      time: this.time,
      klineId: this.klineId,
      open: this.open,
      close: this.close,
      high: this.high,
      low: this.low,
      volume: this.volume,
      deal: this.deal
    }
    let joiValidDepth = Joi.object()
      .keys({
        symbolId: Joi.alternatives().try(Joi.string(), Joi.string().valid('')).required(),
        resolution: Joi.alternatives().try(Joi.string().alphanum()).required(),
        time: Joi.number().integer().min(0).required(),
        klineId: Joi.string().required(),
        open: Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50).required(),
        close: Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50).required(),
        high: Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50).required(),
        low: Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50).required(),
        volume: Joi.alternatives().try(
          Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50),
          Joi.number().integer().min(0)
        ),
        deal: Joi.alternatives().try(
          Joi.string().regex(/^\d+\.\d+$|^\d+$/).min(1).max(50),
          Joi.number().integer().min(0)
        )
      })

    let result = Joi.validate(data, joiValidDepth)
    if (result.error) {
      throw result.error
    }
    return result.value
  }

  static arrayFromDFU (dfu = {}, symbolId, resolution = '1') {
    let { c: close, h: high, l: low, o: open, s: status, t: time, v: volume } = dfu
    if (status !== 'ok') {
      throw status
    }
    let klineList = []
    for (let i = 0; i < close.length; i++) {
      let klineDTO = new KlineDTO(UID.create())
      klineDTO.symbolId = symbolId
      klineDTO.resolution = resolution
      klineDTO.close = close[i].toString()
      klineDTO.high = high[i].toString()
      klineDTO.low = low[i].toString()
      klineDTO.open = open[i].toString()
      klineDTO.time = Number(time[i])
      klineDTO.volume = volume[i]
      klineDTO.klineId = `${klineDTO.symbolId}-${klineDTO.resolution}-${klineDTO.time}`
      klineDTO.validate()
      klineList.push(klineDTO)
    }
    return klineList.sort((a, b) => b.time - a.time)
  }

  /**
   * KlineDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      symbolId: this.symbolId,
      resolution: this.resolution,
      klineId: this.klineId,
      time: this.time,
      open: this.open,
      close: this.close,
      high: this.high,
      low: this.low,
      volume: this.volume,
      deal: this.deal
    }
  }

  /**
   * KlineDTO fromObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @param objectData object
   * @returns KlineDTO
  */
  static fromObject (objectData) {
    let klineDTO = new KlineDTO(UID.create())
    klineDTO.symbolId = objectData.symbolId
    klineDTO.resolution = objectData.resolution.toString()
    klineDTO.time = objectData.time
    klineDTO.klineId = objectData.klineId
    klineDTO.open = objectData.open
    klineDTO.close = objectData.close
    klineDTO.high = objectData.high
    klineDTO.low = objectData.low
    klineDTO.volume = objectData.volume
    klineDTO.deal = objectData.deal
    klineDTO.validate()
    return klineDTO
  }
}

module.exports = KlineDTO

==> jsmex-trader-sdk-1.1.50.tgz: package/lib/data/KlineDTO.js <==
const IDTO = global.IDTO || require('./IDTO')
const Joi = global.Joi || require('../../static_dependencies/joi/joi-browser.min.js')
const UID = global.UID || require('../utils/UID')

/**
 * KlineDTO class
 *
 * @author jsb4ch <jsb4ch@jsmex.io>
 * @since 1.0.0
*/
class KlineDTO extends IDTO {
  constructor (uid) {
    super(uid)

    this.symbolId = ''
    this.resolution = '1'
    this.time = 0

    this.klineId = `${this.symbolId}-${this.resolution}-${this.time}`

    this.open = ''
    this.close = ''
    this.high = ''
    this.low = ''
    this.volume = 0
    this.deal = 0
  }

  /**
   * KlineDTO get symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
   * @return {String}
  */
  get symbol () {
    return this.symbolId
  }

  /**
   * KlineDTO set symbol
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.69
   * @param {String} symbolId
  */
  set symbol (symbolId) {
    this.symbolId = symbolId
  }

  /**
   * KlineDTO validation method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.0
  */
  validate () {
    this.klineId = `${this.symbolId}-${this.resolution}-${this.time}`
    let data = {
      symbolId: this.symbolId,
      resolution: this.resolution,
      time: this.time,
      klineId: this.klineId,
      open: this.open,
      close: this.close,
      high: this.high,
      low: this.low,
      volume: this.volume,
      deal: this.deal
    }
    let joiValidDepth = Joi.object()
      .keys({
        symbolId: Joi.alternatives().try(Joi.string(), Joi.string().valid('')).required(),
        resolution: Joi.alternatives().try(Joi.string().alphanum()).required(),
        time: Joi.number().integer().min(0).required(),
        klineId: Joi.string().required(),
        open: Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50).required(),
        close: Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50).required(),
        high: Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50).required(),
        low: Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50).required(),
        volume: Joi.alternatives().try(
          Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50),
          Joi.number().integer().min(0)
        ),
        deal: Joi.alternatives().try(
          Joi.string().regex(/^\d+\.\d+$|^\d+\.\d+e-\d+$|^\d+$|^\d+e-\d+$/).min(1).max(50),
          Joi.number().integer().min(0)
        )
      })

    let result = Joi.validate(data, joiValidDepth)
    if (result.error) {
      throw result.error
    }
    return result.value
  }

  static arrayFromDFU (dfu = {}, symbolId, resolution = '1') {
    let { c: close, h: high, l: low, o: open, s: status, t: time, v: volume } = dfu
    if (status !== 'ok') {
      throw status
    }
    let klineList = []
    for (let i = 0; i < close.length; i++) {
      let klineDTO = new KlineDTO(UID.create())
      klineDTO.symbolId = symbolId
      klineDTO.resolution = resolution
      klineDTO.close = close[i].toString()
      klineDTO.high = high[i].toString()
      klineDTO.low = low[i].toString()
      klineDTO.open = open[i].toString()
      klineDTO.time = Number(time[i])
      klineDTO.volume = volume[i]
      klineDTO.klineId = `${klineDTO.symbolId}-${klineDTO.resolution}-${klineDTO.time}`
      klineDTO.validate()
      klineList.push(klineDTO)
    }
    return klineList.sort((a, b) => b.time - a.time)
  }

  /**
   * KlineDTO toObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @returns {Object}
  */
  toObject () {
    return {
      uid: this.uid,
      symbolId: this.symbolId,
      resolution: this.resolution,
      klineId: this.klineId,
      time: this.time,
      open: this.open,
      close: this.close,
      high: this.high,
      low: this.low,
      volume: this.volume,
      deal: this.deal
    }
  }

  /**
   * KlineDTO fromObject method
   *
   * @author jsb4ch <jsb4ch@jsmex.io>
   * @since 1.0.14
   * @param objectData object
   * @returns KlineDTO
  */
  static fromObject (objectData) {
    let klineDTO = new KlineDTO(UID.create())
    klineDTO.symbolId = objectData.symbolId
    klineDTO.resolution = objectData.resolution.toString()
    klineDTO.time = objectData.time
    klineDTO.klineId = objectData.klineId
    klineDTO.open = objectData.open
    klineDTO.close = objectData.close
    klineDTO.high = objectData.high
    klineDTO.low = objectData.low
    klineDTO.volume = objectData.volume
    klineDTO.deal = objectData.deal
    klineDTO.validate()
    return klineDTO
  }
}

module.exports = KlineDTO
